#include <htc.h>
#include "../hardware/display.h"
#include "../hardware/buttons.h"
#include "../utility/one-wire.h"
#include "../utility/delay.h"

unsigned char try_read_rom(unsigned char* buffer) {
	if (one_wire_reset()) return 0;
	one_wire_write_byte(ONE_WIRE_COMMAND_READ_ROM);
	for (unsigned char i = 0; i < 8; ++i) {
		buffer[i] = one_wire_read_byte();
	}
	unsigned char crc = one_wire_calculate_crc(buffer, 7);
	return crc == buffer[7];
}

void one_wire_id(void) {
	
	one_wire_init();

	unsigned char rom_id[8];

	while (BUTTON_1 || BUTTON_2) {
		// Try to read the ROM ID.
		if (try_read_rom(rom_id)) {
			// We've read a ROM ID successfully. Display it in four sections.
			for (unsigned char i = 0; i < 4 && (BUTTON_1 || BUTTON_2); ++i) {
				// Wait for button 1 to be released before displaying part of the ID.
				while (!BUTTON_1 && BUTTON_2);
				// Display a part of the ID.
				display_print_hex_word((rom_id[i * 2] << 8) | rom_id[i * 2 + 1]);
				// Display a decimal point to show which part we're showing (part n of 4).
				display_data[i] |= 0x80;
				// Wait for button 1 to be pressed to advance to the next digit.
				delay_ms(10);
				while (BUTTON_1);
				delay_ms(10);
			}
		} else {
			// Display '----'.
			display_set_display(0b01000000, 0b01000000, 0b01000000, 0b01000000);
		}
	}
}