#include <htc.h>

#include "../modes/pulser.h"
#include "../hardware/display.h"
#include "../hardware/buttons.h"
#include "../hardware/resistors.h"
#include "../main/settings.h"
#include "../utility/delay.h"

extern void tmr2_set_postscale(unsigned char postscale);

void display_pulser_rate(void) {
	const unsigned char rate_strings[] = {
		0b00000000, 0b10111111, 0b01101101, // 0.5
		0b00000000, 0b00000000, 0b00000110, //   1
		0b00000000, 0b00000000, 0b01011011, //   2
		0b00000000, 0b00000000, 0b01101101, //   5
		0b00000000, 0b00000110, 0b00111111, //  10
		0b00000000, 0b01101101, 0b00111111, //  50
		0b00000110, 0b00111111, 0b00111111, // 100
		0b01101101, 0b00111111, 0b00111111, // 500
		0b00000000, 0b00000110, 0b01110110, //  1K

	};
	const unsigned char* src = rate_strings + (settings.pulser_rate * 3);
	for (unsigned char d = 1; d < 4; ++d) {
		display_data[d] = *(src++);
	}
}

void pulser(void) {

	display_clear();
	display_pulser_rate();

	unsigned char previous_level;

	const unsigned int timer_periods[] = {
		4000,
		2000,
		1000,
		 400,
		 200,
		  40,
		  20,
		   4,
		   2,
	};

	while (BUTTON_1 || BUTTON_2) {

		// Sample the current probe tip signal level.
		previous_level = R20;

		if (previous_level) {
			display_data[0] = 0b00000001; // Overline
		} else {
			display_data[0] = 0b00001000; // Underline.
		}

		if (!BUTTON_1 && BUTTON_2) {
			
			display_data[0] |= 0b01000000; // Middle line.
			delay_ms(1);

			// Use TMR2 to time the pulses.
			TMR2ON = 1;				
			// Use a prescaler of 1:4.
			// This results in a timer frequency of (Fosc/4)/4 = (20MHz/4)/4 = 1.25MHz.
			T2CKPS1 = 0; T2CKPS0 = 1;
			// Set the period of the counter to 100us.
			PR2 = 124; // (1.25MHz * 100us) - 1

			// Set the postscaler to 5 to have an interrupt period of 500us.
			tmr2_set_postscale(5);

			// How many 1ms timer ticks do we have to wait?
			unsigned int timer_period = timer_periods[settings.pulser_rate], timer_ticks = 0;

			// Interrupts mess up timing.
			di();
			
			// Reset TMR2 before starting.
			TMR2 = 0;

			while (!BUTTON_1 && BUTTON_2) {
				TMR2IF = 0;
				while (!TMR2IF);
				if (timer_ticks == 0) {
					// Output a pulse and reset the timer.
					R20 = !previous_level;
					TRIS_R20 = 0;
					_delay(1);
					TRIS_R20 = 1;
					timer_ticks = timer_period;
				}
				// Refresh the display in the background.
				display_output_segment();					
				--timer_ticks;
			}
			
			TMR2ON = 0;
			ei();
		}

		if (!BUTTON_2 && BUTTON_1) {
			++settings.pulser_rate;
			if (settings.pulser_rate >= PULSER_RATES_COUNT) {
				settings.pulser_rate = 0;
			}
			display_pulser_rate();
			delay_ms(1);
			while (!BUTTON_2 && BUTTON_1);
			settings_save();
		}
	}
}